// Dynamic link library implementation of NeuroSolutions Momentum component

#include "NSDLL.h"

/*****************************/
/* Gradient search procedure */

__declspec(dllexport) void performMomentum(
	DLLData	*instance,	// Pointer to instance data (may be NULL)
	NSFloat	*weights,	// Pointer to the vector of weights
	int	length,		// Length of the weight vector
	NSFloat	*gradient,	// Pointer to the vector of gradients, one for each weight
	NSFloat	*step,		// Pointer to the learning rate/s
	BOOL	individual,	// Indicates whether their is one learning rate for all weights (FALSE),
				// or each weight has its own learning rate
	int		stepDivisor,// The number each step size should be divided by
	BOOL	decayWeights,	// TRUE if weight decay is active
	NSFloat decayRate,	// Rate to decay weights if weight decay is active
	NSFloat	momentum,	// Momentum rate for all weights
	NSFloat	*delta		// Last weight Update
	)
{
	register int i;

	if (decayWeights) {
		for (i=0; i<length; i++) {
			delta[i] = momentum*delta[i] + (step[individual?i:0]/stepDivisor)*gradient[i];
			weights[i] += delta[i] - decayRate * weights[i];
		}
	}
	else {
		for (i=0; i<length; i++)
			weights[i] += delta[i] = momentum*delta[i] + (step[individual?i:0]/stepDivisor)*gradient[i];
	}
}

/******************************************/
/* Management of instance data (OPTIONAL) */
/*
__declspec(dllexport) DLLData *allocMomentum(
	DLLData	*oldInstance,	// Pointer to the last instance if reallocating
	int	length,		// Length of the weight vector
	BOOL	individual	// Indicates whether their is one learning rate for all weights (FALSE),
				// or each weight has its own learning rate
	)
{
	DLLData *instance = allocDLLInstance(oldInstance);
	return instance;
}

__declspec(dllexport) void freeMomentum(DLLData *instance)
{
	freeDLLInstance(instance);
}
*/
